//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 08/09/02; 15:10:44

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "graphic.h"
#include "menulib.h"
#include "tictac.h"

//---------------------------------------------------------------------------

extern TMenu * Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//-----------------------------------------------------------------------------

char * yesno_str[] = {"No","Yes"};


char * Plane0;
char * Plane1;

TGame*Game;

//---------------------------------------------------------------------------

void NewGame(void)
{
	int a,b;
	
	
	for (a=0;a<Game->Dim.x;a++)
	for (b=0;b<Game->Dim.y;b++)
	Game->Board.Matrix[a][b] = TT_NONE;
	
	
	Game->CursorPos.x = Game->Dim.x/2;
	Game->CursorPos.y = Game->Dim.y/2;
	
	Game->History.step_max = 0;
	Game->History.step = 0;
	
	Game->Board.turn = 0;
	Game->Board.state = TG_PLAYING;
	
	
}
//-----------------------------------------------------------------------------

void RefreshScreen(void)
{
	int a,b;
	char buffer[50];
	
	ClearGrayScreen2B(Plane0,Plane1);
	
	// TILES
	
	for (a=0;a<Game->Dim.x;a++)
	for (b=0;b<Game->Dim.y;b++)
	{
		char tile_num=Game->Board.Matrix[a][b];
		GraySprite8_OR((a<<3),(b<<3),8,
		Image_C0+(tile_num<<3),
		Image_C1+(tile_num<<3),
		Plane0,Plane1);
	}
	
	// STATS
	
	
	FontSetSys(0);
	int stats_top=0;
	
	sprintf(buffer,"TicTacToe");
	DrawGrayStr2B(STATS_LEFT,stats_top,buffer,A_REPLACE,Plane0,Plane1);
	stats_top+=10;
	
	sprintf(buffer,"Player : %d",Game->Board.turn+1);
	DrawGrayStr2B(STATS_LEFT,stats_top,buffer,A_REPLACE,Plane0,Plane1);
	stats_top+=10;
	
	if (Game->Board.state==TG_WIN)
	sprintf(buffer,"Player Wins!");
	else if (Game->Board.state==TG_EXAEQUO)
	sprintf(buffer,"Ex Aequo!");
	else
	strcpy(buffer,"");
	
	DrawGrayStr2B(STATS_LEFT,stats_top,buffer,A_REPLACE,Plane0,Plane1);
	stats_top+=10;
	
	FontSetSys(1);
	
	
	FastCopyScreen(Plane0,GetPlane(0));
	FastCopyScreen(Plane1,GetPlane(1));
}
//-----------------------------------------------------------------------------


void Play(TPoint Point)
{
	
	if (Game->Board.state != TG_PLAYING)
	return;
	
	
	if (Game->fall_mode)
	{
		Point.y = Game->Dim.y-1;
		while (Game->Board.Matrix[Point.x][Point.y]!=TT_NONE)
		Point.y--;
	}
	
	Game->Board.Matrix[Point.x][Point.y] = Game->Board.turn?TT_PLAYER2:TT_PLAYER1;
	Game->Board.turn^=1;
	
	
	Game->History.History[Game->History.step] = Point;
	Game->History.step++;
	Game->History.step_max = Game->History.step;
	
	
	if (Has_Won(Point))
	Game->Board.state = TG_WIN;
	else if (Game->History.step==(Game->Dim.x*Game->Dim.y))
	Game->Board.state = TG_EXAEQUO;
	else
	Game->Board.state = TG_PLAYING;
	
	
}
//-----------------------------------------------------------------------------


int CanMakeMove(TPoint Position)
{
	int ypos=Game->fall_mode?0:Position.y;
	return (Game->Board.Matrix[Position.x][ypos]==TT_NONE);
}
//-----------------------------------------------------------------------------

int Has_Won(TPoint Position)
{
	int a,b;
	int count;
	int PlayerTurn = Game->Board.Matrix[Position.x][Position.y];
	TPoint Position_Connect;
	
	int orientations[4][2] = {{1,0},{0,1},{1,1},{1,-1}};
	
	
	for (a=0;a<4;a++)
	{
		
		count = 0;
		
		for (b=-(Game->length_to_connect-1);b<=(Game->length_to_connect-1);b++)
		{
			Position_Connect.x = Position.x+b*orientations[a][0];
			Position_Connect.y = Position.y+b*orientations[a][1];
			
			if (Position_Connect.x>=0 && Position_Connect.x<Game->Dim.x && 
			Position_Connect.y>=0 && Position_Connect.y<Game->Dim.y)
			{
				if (Game->Board.Matrix[Position_Connect.x][Position_Connect.y] == PlayerTurn)
				count++;
				else
				count = 0;
			}
			
			if (count==Game->length_to_connect)
			return 1;
		}
		
	}
	
	return 0;
}
//---------------------------------------------------------------------------

void Undo(void)
{
	
	if (Game->History.step>0)
	{
		Game->History.step--;
		
		Game->CursorPos = Game->History.History[Game->History.step];
		Game->Board.Matrix[Game->CursorPos.x][Game->CursorPos.y] = TT_NONE;
		Game->Board.turn^=1;
		Game->Board.state = TG_PLAYING;
	}
	
}
//-----------------------------------------------------------------------------

void Redo(void)
{
	
	if (Game->History.step<Game->History.step_max)
	{
		Game->CursorPos = Game->History.History[Game->History.step];
		
		Game->Board.Matrix[Game->CursorPos.x][Game->CursorPos.y] = Game->Board.turn?TT_PLAYER2:TT_PLAYER1;
		Game->Board.turn^=1;
		
		Game->History.step++;
		
		
	}
	
}
//-----------------------------------------------------------------------------

void PlayRandomPos(void)
{
	TPoint RandomPos;
	do
	{
		RandomPos.x=random(Game->Dim.x);
		RandomPos.y=random(Game->Dim.y);
	}
	while (!CanMakeMove(RandomPos));
	Play(RandomPos);
	Game->CursorPos=RandomPos;
}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{
	short key;
	char buffer[50];
	
	if (!(Plane0=malloc(2*LCD_SIZE)))
	goto end_prog;
	Plane1=Plane0+LCD_SIZE;
	
	TMenu MenuA;
	Menu = &MenuA;
	
	TGame GameA;
	Game=&GameA;
	
	randomize();
	
	
	
	Game->Dim.x = 12;
	Game->Dim.y = 12;
	Game->length_to_connect = 5;
	Game->fall_mode = 0;
	
	for (;;)
	{
		
		
		MenuNew2();
		MenuAddTitle("TicTacToe");
		MenuAddItem("Play");
		MenuAddItem("Options");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		MenuDo(ME_MAIN);
		
		if (!Menu->ok) break;
		
		if (Menu->Index==1)
		{
			
			GrayOn();
			
			NewGame();
			RefreshScreen();
			
			for (;;)
			{
				
				GraySprite8_XOR((Game->CursorPos.x<<3),(Game->CursorPos.y<<3),8,
				TileBlack,TileBlack,GetPlane(0),GetPlane(1));
				key = ngetchx();
				GraySprite8_XOR((Game->CursorPos.x<<3),(Game->CursorPos.y<<3),8,
				TileBlack,TileBlack,GetPlane(0),GetPlane(1));
				
				
				if (key==KEY_LEFT)
				{
					Game->CursorPos.x--;
					if (Game->CursorPos.x<0)
					Game->CursorPos.x = (Game->Dim.x-1);
				}
				
				else if (key==KEY_RIGHT)
				{
					Game->CursorPos.x++;
					if (Game->CursorPos.x>(Game->Dim.x-1))
					Game->CursorPos.x = 0;
				}
				
				else if (key==KEY_UP)
				{
					Game->CursorPos.y--;
					if (Game->CursorPos.y<0)
					Game->CursorPos.y = (Game->Dim.y-1);
				}
				
				else if (key==KEY_DOWN)
				{
					Game->CursorPos.y++;
					if (Game->CursorPos.y>(Game->Dim.y-1))
					Game->CursorPos.y = 0;
				}
				
				else if (key==KEY_F1)
				{
					
					if (CanMakeMove(Game->CursorPos))
					{
						Play(Game->CursorPos);
						RefreshScreen();
					}
					
				}
				
				else if (key==KEY_F2)
				{
					NewGame();
					RefreshScreen();
				}
				
				else if (key==KEY_F3)
				{
					PlayRandomPos();
					RefreshScreen();
				}
				
				else if (key==KEY_BACKSPACE)
				{
					Undo();
					RefreshScreen();
				}
				
				else if (key==KEY_CLEAR)
				{
					Redo();
					RefreshScreen();
				}
				
				else if (key==KEY_APPS)
				{
					off();
				}
				
				else if (key==KEY_ESC)
				{
					
					GrayOff();
					clrscr();
					printf("[ESC]   : Continue \n");
					printf("[ENTER] : Exit \n");
					while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
					GrayOn();
					
					if (key==KEY_ENTER)
					break;
					
					RefreshScreen();
				}
				
			}
			
			GrayOff();
			
		}
		
		else if (Menu->Index==2)
		{
			
			for (;;)
			{
				
				MenuNew2();
				MenuAddTitle("Options");
				sprintf(buffer,"Connect : %d",Game->length_to_connect);
				MenuAddItem(buffer);
				sprintf(buffer,"Width : %d",Game->Dim.x);
				MenuAddItem(buffer);
				sprintf(buffer,"Height : %d",Game->Dim.y);
				MenuAddItem(buffer);
				sprintf(buffer,"Fall Mode : %s",yesno_str[Game->fall_mode]);
				MenuAddItem(buffer);
				
				MenuDo(ME_OPTIONS);
				
				if (!Menu->ok) break;
				
				if (Menu->Index==1)
				{
					Game->length_to_connect++;
					if (Game->length_to_connect>6)
					Game->length_to_connect = 3;
				}
				else if (Menu->Index==2)
				{
					Game->Dim.x++;
					if (Game->Dim.x>12)
					Game->Dim.x = 3;
				}
				else if (Menu->Index==3)
				{
					Game->Dim.y++;
					if (Game->Dim.y>12)
					Game->Dim.y = 3;
				}
				else if (Menu->Index==4)
				{
					Game->fall_mode^=1;
				}
				
			}
			
		}
		
		else if (Menu->Index==3)
		{
			clrscr();
			FontSetSys(0);
			printf("About \n");
			printf("TicTacToe 1.3 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
		}
		
		else if (Menu->Index==4)
		{
			break;
		}
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
	
}
